//--------------------------------------------------------------
// Press F1 to get help about using script.
// To access an object that is not located in the current class, start the call with Globals.
// When using events and timers be cautious not to generate memoryleaks,
// please see the help for more information.
//---------------------------------------------------------------
	
    /*
	*******************************************************************
	All examples in this support document are only intended
	to improve understanding of the functionality and 
	handling of the equipment. 
	Beijer Electronics AB cannot assume any liability if these
	examples are used in real applications.
	In view of the wide range of applications for this 
	equipment, users must acquire sufficient knowledge 
	themselves in order to ensure that it is correctly used in 
	their specific application. Persons responsible for the 
	application and the equipment must themselves 
	ensure that each application is in compliance with all 
	relevant requirements, standards and legislation in 
	respect to configuration and safety.
	Beijer Electronics AB will accept no liability for any 
	damage incurred during the installation or use of this 
	equipment.
	Beijer Electronics AB prohibits all modification, changes
	or conversion of the equipment.
	*******************************************************************
	*/

namespace Neo.ApplicationFramework.Generated
{
	using System.Windows.Forms;
	using System;
	using System.Drawing;
	using Neo.ApplicationFramework.Tools;
	using Neo.ApplicationFramework.Common.Graphics.Logic;
	using Neo.ApplicationFramework.Controls;
	using Neo.ApplicationFramework.Interfaces;
	using System.Data;
	using System.Data.SqlServerCe;
	using System.IO;
	using System.Reflection;
	using System.Collections.Generic;


	public partial class DatabaseFunctions
	{
		/// <summary>
		/// Member variable to hold a sql connection.
		/// </summary>
		private SqlCeConnection m_SqlConnection;
		
		public DataTable GetTableFromDB(string tableName)
		{
			DataTable table = GetTable(tableName);
			return table;
		}
		
		//Returns a list with all tables in Database.sdf
		public List<string> GetTableNames()
		{
			DataTable table = new DataTable();
			List<string> tableNames = new List<string>();
			if (ConnectToDataBase())
			{
				// Using try/catch to prevent a crash, should anything go wrong.
				try
				{
					// This is the sql command to excecute. Fetch all data from the given table.
					string sqlString = string.Format("select table_name from information_schema.tables where TABLE_TYPE <> 'VIEW' ");
					// Create a data adapter to excecute the query and fetch the data.
					// It uses the connection variable.
					SqlCeDataAdapter sqlCeDataAdapter = new SqlCeDataAdapter(sqlString, m_SqlConnection);
					sqlCeDataAdapter.Fill(table);
					
					foreach (DataRow dataRow in table.Rows)
					{
						tableNames.Add(dataRow.ItemArray.GetValue(0).ToString());
					}
				}
				catch (SqlCeException)
				{
					// Catches any exceptions from the database and prevents the 
					// program from terminating unexpectedly.
				}
			}
			// Disconnects from the database.
			// This is very important.
			// If you leave connections open, there can be unexpected behaviour,
			// like locks and crashes. There is also a memory cost for every open
			// connection.
			DisconnectFromDataBase();
			// Return the result.
			return tableNames;
		}
		
		//Returns a datatable from the database.sdf
		private DataTable GetTable(string tableName)
		{
			// Variable to hold the data.
			DataTable table = new DataTable();
			// Call a method that tries to connect to the database.
			if (ConnectToDataBase())
			{
				// Using try/catch to prevent a crash, should anything go wrong.
				try
				{
					// This is the sql command to excecute. Fetch all data from the given table.
					string sqlString = string.Format("SELECT * FROM {0}", tableName);
					// Create a data adapter to excecute the query and fetch the data.
					// It uses the connection variable.
					SqlCeDataAdapter sqlCeDataAdapter = new SqlCeDataAdapter(sqlString, m_SqlConnection);
					// Populate the DataSet with the result from the data adapter.
					sqlCeDataAdapter.Fill(table);
				}
				catch (SqlCeException)
				{
					// Catches any exceptions from the database and prevents the 
					// program from terminating unexpectedly.
				}
			}
			// Disconnects from the database.
			// This is very important.
			// If you leave connections open, there can be unexpected behaviour,
			// like locks and crashes. There is also a memory cost for every open
			// connection.
			DisconnectFromDataBase();

			// Return the result.
			return table;
		}

		/// <summary>
		/// Tries to connect to the database.
		/// </summary>
		/// <returns>True if successful, otherwise false.</returns>
		private bool ConnectToDataBase()
		{
			// Checks if there already is an active connection. 
			// If so, it returns true right away.
			if (m_SqlConnection != null && m_SqlConnection.State == ConnectionState.Open)
				return true;

			// Create a new sql connection object.
			m_SqlConnection = new SqlCeConnection();
			// Specify the connectionstring, with the full path to the database.
			m_SqlConnection.ConnectionString = string.Format("data source={0}", Path.Combine(ExcecutingPath, "Database.sdf"));

			// Using a try/catch in case something goes wrong.
			// Should the full path to the database be wrong, it will throw an exception.
			// This prevents the termination of the program.
			try 
			{
				// Tries to open the connection.
				m_SqlConnection.Open();
			}
			catch (SqlCeException)
			{
				// If it went wrong, terminate the connection completely.
				DisconnectFromDataBase();
				// Return false to indicate that the connection couldn't be made.
				return false;
			}

			// Checks the state of the connection, returning true if it is open.
			return m_SqlConnection.State == ConnectionState.Open;
		}

		/// <summary>
		/// Disconnects from the database and disposes the connection.
		/// </summary>
		private void DisconnectFromDataBase()
		{
			// Makes sure that the connection isn't already disposed.
			if (m_SqlConnection != null)
			{
				// Checks to see if the connection is open.
				if(m_SqlConnection.State == ConnectionState.Open)
				{
					// Close the connection.
					m_SqlConnection.Close();
				}
				// Dispose the connection object.
				m_SqlConnection.Dispose();
				// Set it to null, so that it is easy to find out
				// if it is in use or not.
				m_SqlConnection = null;
			}
		}
		
		/// <summary>
		/// Get the path to where the project is excecuting.
		/// This differs if you run the project from the pc
		/// or in the terminal.
		/// </summary>
		private string ExcecutingPath
		{
			get { return Path.GetDirectoryName(Assembly.GetExecutingAssembly().ManifestModule.FullyQualifiedName); }
		}
	}
}
